import 'package:test/test.dart';
import 'package:dynamic_middleware/dynamic_middleware.dart';
import 'package:redux/redux.dart';

/// Simple reducer to increment on every action.
dynamic get reducer => (state, action) => state + 1;

void main() {
  Store<int> store, storeFromList;
  DynamicMiddleware<int> dynamicMiddleware, dynamicMiddlewareFromList;

  var count1, count2;
  void middleware1(Store<int> store, Object action, NextDispatcher next) {
    count1++;
    next(action);
  }
  void middleware2(Store<int> store, Object action, NextDispatcher next) {
    count2++;
    next(action);
  }

  setUp(() {
    count1 = 0;
    count2 = 0;

    dynamicMiddleware = new DynamicMiddleware<int>.empty();
    dynamicMiddlewareFromList = new DynamicMiddleware<int>.fromList(<Middleware<int>>[middleware1, middleware2]);
    store = new Store<int>(reducer,
        middleware: <Middleware<int>>[dynamicMiddleware],
        initialState: 0);
    storeFromList =  new Store<int>(reducer,
        middleware: <Middleware<int>>[dynamicMiddlewareFromList],
        initialState: 0);
  });

  test('redux should work without error', () {
    expect(store.state, equals(0));
    store.dispatch('increment');
    expect(store.state, equals(1));
  });

  test('added middleware should be called on every dispatch', () {
    dynamicMiddleware.add(middleware1);
    store.dispatch('increment');
    expect(count1, equals(1));
    store.dispatch('increment');
    expect(count1, equals(2));
  });

  test('multiple added middlewares should be called', () {
    dynamicMiddleware
        ..add(middleware1)
        ..add(middleware2);

    store.dispatch('increment');
    expect(count1, equals(1));
    expect(count2, equals(1));
  });

  test('all middlewares added by addAll should be called', () {
    dynamicMiddleware.addAll([middleware1, middleware2]);
    store.dispatch('increment');
    expect(count1, equals(1));
    expect(count2, equals(1));
  });

  test('all middlewares added by toList constructor should be called', () {
    storeFromList.dispatch('increment');
    expect(count1, equals(1));
    expect(count2, equals(1));
  });

  test('removed middlewares should not be called', () {
    dynamicMiddleware
        ..addAll([middleware1, middleware2])
        ..remove(middleware2);
    store.dispatch('increment');
    expect(count1, equals(1));
    expect(count2, equals(0));
  });

  test('middlewares should not be called after clearing', () {
    dynamicMiddleware
        ..addAll([middleware1, middleware2])
        ..clear();
    store.dispatch('increment');
    expect(count1, equals(0));
    expect(count2, equals(0));
  });
}